@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # Main.py
-from fabric_quick_setup.mod import *
+from fabric_quick_setup.mod import Mod, InvalidModResourceError, ModVersionNotFoundError
 import json
 import os
 import platform
@@ -62,11 +62,23 @@ def clean_exit(color: str):
     log.print_log('Exiting...', color)
     sys.exit()
 
-def install_fabric(mc_dir, mc_version, server): 
+def ensure_dir(path):
+    if not os.path.exists(path):
+        os.makedirs(path)
+
+def download_fabric_installer(dir):
+    fabric_installers = requests.get('https://meta.fabricmc.net/v2/versions/installer').json()
+    r = requests.get(fabric_installers[0]['url'])
+    filename = f'{dir}\\fabric-installer.jar'
+    with open(filename, 'wb') as outfile:
+        outfile.write(r.content)
+    return filename
+
+def install_fabric(installer_path, mc_dir, mc_version, server): 
     if server:
         installer = subprocess.run(['java', 
             '-jar', 
-            'fabric-installer.jar', 
+            installer_path, 
             'server',
             '-snapshot',
             '-mcversion',
@@ -79,7 +91,7 @@ def install_fabric(mc_dir, mc_version, server):
     else:
         installer = subprocess.run(['java', 
             '-jar', 
-            'fabric-installer.jar', 
+            installer_path, 
             'client',
             '-snapshot',
             '-noprofile',
@@ -208,13 +220,14 @@ def ask_mods(mods):
 
 @click.command()
 @click.option('--debug', is_flag=True, default=False)
+@click.option('--mod-list', 'mod_list_url', default='https://raw.githubusercontent.com/max-niederman/fabric-quick-setup/master/fabric_quick_setup/mods.json', type=str, help='Mod list URL.')
+@click.option('--installer', 'installer_path', type=click.Path(), help='Path to Fabric Installer')
 @click.option('-s', '--server', is_flag=True, default=False)
-@click.option('-u', '--mod-list', 'mod_list_url', default='https://raw.githubusercontent.com/max-niederman/fabric-quick-setup/master/fabric_quick_setup/mods.json', type=str, help='Mod list URL.')
 @click.option('-d', '--mc-dir', type=click.Path(), help='Minecraft directory')
 @click.option('-t', '--mc-modded-dir', type=click.Path(), help='Minecraft modded directory')
 @click.option('-v', '--version', 'mc_version', type=str, help='Minecraft version to install')
-@click.option('-m', '--mods', 'mod_ids', type=str, multiple=True, help='The person to greet.')
-def main(debug, server, mod_list_url, mc_dir, mc_modded_dir, mc_version, mod_ids):
+@click.option('-m', '--mods', 'mod_ids', type=str, multiple=True, help='Mods to install. Use this once for each mod.')
+def main(debug, mod_list_url, installer_path, server, mc_dir, mc_modded_dir, mc_version, mod_ids):
     """
     CLI to install Fabric Loader and Popular Mods
     """
@@ -230,6 +243,10 @@ def main(debug, server, mod_list_url, mc_dir, mc_modded_dir, mc_version, mod_ids
     if not mc_dir:
         mc_dirs = ask_mc_dirs(mc_default_path, server)
         mc_dir, mc_modded_dir = mc_dirs['mc_dir'], mc_dirs['mc_modded_dir']
+    
+    ensure_dir(mc_dir)
+    ensure_dir(mc_modded_dir)
+    ensure_dir(f'{mc_modded_dir}\mods')
 
     if not mc_version:
         mc_version = ask_version()
@@ -247,7 +264,7 @@ def main(debug, server, mod_list_url, mc_dir, mc_modded_dir, mc_version, mod_ids
     if debug:
         mod_list = json.load(open('mods.json'))
     else:
-        mod_list = requests.get(mod_list_url).json()
+        mod_list = json.loads(requests.get(mod_list_url).content)
     
     if not mod_ids:
         mod_names = ask_mods(mod_list)
@@ -256,9 +273,13 @@ def main(debug, server, mod_list_url, mc_dir, mc_modded_dir, mc_version, mod_ids
     mod_ids.update(resolve_dependencies(mod_ids, mod_list))
     mods = [mod for mod in mod_list if mod['id'] in mod_ids]
     
-    log.print_log('Beginning setup: Installing Fabric Loader', 'green')
-    
-    install_fabric(mc_dir, mc_version, server)
+    if not installer_path:
+        log.print_log('Beginning setup: Downloading Fabric Installer', 'green')
+        installer_path = download_fabric_installer(mc_dir)
+        log.print_log('Finished Downloading Fabric Installer.', 'green')
+
+    log.print_log('Starting Fabric Loader installation.', 'green')
+    install_fabric(installer_path, mc_dir, mc_version, server)
     log.print_log('Finished Fabric Loader installation.', 'green')
 
     log.print_log('Starting mod installation. This may take a while...', 'green')
